@@ -8,11 +8,17 @@ use base 'My::Model';
 __PACKAGE__->set_up_table('cities');
 
 
-__PACKAGE__->has_a(
+__PACKAGE__->belongs_to(
   state =>
     'My::State' =>
       'state_id'
 );
 
+__PACKAGE__->has_one(
+  zipcode =>
+    'My::Zipcode' =>
+      'city_id'
+);
+
 1;# return true:
 
